/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import org.onosproject.nemo.model.common.UserId;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * Representation of the beginning transaction input builder.
 */
public class BeginTransactionInputBuilder {

    private Map<Class<? extends BeginTransactionInput>, BeginTransactionInput>
            augmentation = new HashMap<>();

    private UserId userId;

    /**
     * Creates a beginning transaction input builder.
     */
    public BeginTransactionInputBuilder() {
    }

    /**
     * Creates a beginning transaction input builder from
     * a beginning transaction input.
     *
     * @param base a beginning transaction input builder object
     */
    public BeginTransactionInputBuilder(BeginTransactionInput base) {
        userId = base.getUserId();
        if (base instanceof BeginTransactionInputImpl) {
            BeginTransactionInputImpl impl = (BeginTransactionInputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the user ID for the builder to use.
     *
     * @param id user's ID to set
     * @return self
     */
    public BeginTransactionInputBuilder userId(UserId id) {
        userId = id;
        return this;
    }

    /**
     * Adds augmentation by a class type and a beginning transaction input.
     * If the augmentation is null, remove the augmentation type
     * key from the hash map.
     *
     * @param augmentationType class extends BeginTransactionInput
     * @param augmentation a beginning transaction input object
     * @return self
     */
    public BeginTransactionInputBuilder addAugmentation(
            Class<? extends BeginTransactionInput> augmentationType,
            BeginTransactionInput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType class extends BeginTransactionInput
     * @return self
     */
    public BeginTransactionInputBuilder removeAugmentation(
            Class<? extends BeginTransactionInput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a beginning transaction input object.
     *
     * @return beginning transaction input object
     */
    public BeginTransactionInput build() {
        return new BeginTransactionInputImpl(this);
    }

    /**
     * Representation of the implement of BeginTransactionInput.
     */
    private static final class BeginTransactionInputImpl
            implements BeginTransactionInput {

        private final UserId userId;

        private final Map<Class<? extends BeginTransactionInput>,
                BeginTransactionInput> augmentation;

        private BeginTransactionInputImpl(BeginTransactionInputBuilder base) {
            userId = base.userId;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public UserId getUserId() {
            return userId;
        }

        @Override
        public int hashCode() {
            return Objects.hash(userId, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            BeginTransactionInputImpl other = (BeginTransactionInputImpl) obj;
            return Objects.equals(userId, other.getUserId()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("userId", userId)
                    .add("augmentation", augmentation).toString();
        }
    }
}