/*
 * Copyright 2014-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.instance.parameter.values;

import static com.google.common.base.MoreObjects.toStringHelper;
import static org.onosproject.nemo.model.operation.action.instance
                .parameter.values.Utils.checkOrderRange;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
/**
 * The description of builder for integer value objects.
 */
public class IntValueBuilder {

    private final Map<Class<? extends IntValue>, IntValue> augmentation =
            new HashMap<>();

    private IntValueKey key;
    private long order;
    private long value;

    /**
     * Creates an instance by default.
     */
    public IntValueBuilder() {
    }

    /**
     * Creates an integer value builder from a given integer value.
     *
     * @param base integer value
     */
    public IntValueBuilder(IntValue base) {
        if (base.getKey() == null) {
            key = new IntValueKey(base.getOrder(), base.getValue());
            order = base.getOrder();
            value = base.getValue();
        } else {
            key = base.getKey();
            order = key.getOrder();
            value = key.getValue();
        }
    }

    /**
     * Sets the key to be used by the builder.
     *
     * @param value key value
     * @return self
     */
    public IntValueBuilder key(IntValueKey value) {
        key = value;
        return this;
    }

    /**
     * Sets the order to be used by the builder.
     *
     * @param value order value
     * @return self
     */
    public IntValueBuilder order(long value) {
        checkOrderRange(value);
        order = value;
        return this;
    }

    /**
     * Sets the value to be used by the builder.
     *
     * @param value the value
     * @return self
     */
    public IntValueBuilder value(long value) {
        this.value = value;
        return this;
    }

    /**
     * Adds the augmentation to be used by the builder.
     * Remove the augmentation, if the integer value is null.
     *
     * @param augmentationType augmentation type
     * @param augmentation the augmentation to be used
     * @return self
     */
    public IntValueBuilder addAugmentation(
            Class<? extends IntValue> augmentationType,
            IntValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation to be used by integer value builder.
     *
     * @param augmentationType remove augmentation
     * @return an integer value builder
     */
    public IntValueBuilder removeAugmentation(
            Class<? extends IntValue> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a new integer value instance.
     *
     * @return the integer value instance
     */
    public IntValue build() {
        return new IntValueImpl(this);
    }

    /**
     * The implementation of integer value to create an instance.
     */
    private static final class IntValueImpl implements IntValue {

        private final IntValueKey key;
        private final long order;
        private final long value;
        private Map<Class<? extends IntValue>, IntValue> augmentation;

        private IntValueImpl(IntValueBuilder base) {
            if (base.key == null) {
                key = new IntValueKey(base.order, base.value);
                order = base.order;
                value = base.value;
            } else {
                key = base.key;
                order = key.getOrder();
                value = key.getValue();
            }
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public IntValueKey getKey() {
            return key;
        }

        @Override
        public long getOrder() {
            return order;
        }

        @Override
        public long getValue() {
            return value;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, order, value, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            IntValue other = (IntValue) obj;
            if (!Objects.equals(key, other.getKey())) {
                return false;
            }
            if (!Objects.equals(order, other.getOrder())) {
                return false;
            }
            if (!Objects.equals(value, other.getValue())) {
                return false;
            }
            // Simple case: we are comparing against self
            IntValueImpl otherImpl = (IntValueImpl) obj;
            if (!Objects.equals(augmentation, otherImpl.augmentation)) {
                return false;
            }
            return true;
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("order", order)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}