/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.instance.parameter.values;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;
import static org.onosproject.nemo.model.operation.action.instance
                .parameter.values.Utils.checkOrderRange;

/**
 * Builder of string value.
 */
public class StringValueBuilder {

    private Map<Class<? extends StringValue>, StringValue> augmentation =
            new HashMap<>();
    private StringValueKey key;
    private long order;
    private String value;

    /**
     * Creates object by default.
     */
    public StringValueBuilder() {
    }

    /**
     * Creates a copy from Source Object.
     *
     * @param base string value
     */
    public StringValueBuilder(StringValue base) {
        if (base.getKey() == null) {
            key = new StringValueKey(base.getOrder(), base.getValue());
            order = base.getOrder();
            value = base.getValue();
        } else {
            key = base.getKey();
            order = key.getOrder();
            value = key.getValue();
        }
    }

    /**
     * Sets key of instance.
     *
     * @param value for the key
     * @return self
     */
    public StringValueBuilder key(StringValueKey value) {
        key = value;
        return this;
    }

    /**
     * Sets order of instance.
     *
     * @param order for the order
     * @return self
     */
    public StringValueBuilder order(long order) {
        checkOrderRange(order);
        this.order = order;
        return this;
    }

    /**
     * Sets value of the instance.
     *
     * @param value to be set
     * @return self
     */
    public StringValueBuilder value(String value) {
        this.value = value;
        return this;
    }

    /**
     * Adds augmentation for the instance.
     * Remove the augmentation, if the string value is null.
     *
     * @param augmentationType class of string value
     * @param augmentation string value
     * @return self
     */
    public StringValueBuilder addAugmentation(
            Class<? extends StringValue> augmentationType,
            StringValue augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation for the instance.
     *
     * @param augmentationType augmentation to be removed
     * @return builder of string value
     */
    public StringValueBuilder removeAugmentation(
            Class<? extends StringValue> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds the string value instance.
     *
     * @return new string value instance
     */
    public StringValue build() {
        return new StringValueImpl(this);
    }

    /**
     * The implementation of string value to create an instance.
     */
    private static final class StringValueImpl implements StringValue {

        private final StringValueKey key;
        private final long order;
        private final String value;
        private final Map<Class<? extends StringValue>, StringValue> augmentation;

        /**
         * Creates a copy from Source Object.
         *
         * @param base Source object
         */
        private StringValueImpl(StringValueBuilder base) {
            if (base.key == null) {
                key = new StringValueKey(base.order, base.value);
                order = base.order;
                value = base.value;
            } else {
                key = base.key;
                order = key.getOrder();
                value = key.getValue();
            }
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public StringValueKey getKey() {
            return key;
        }

        @Override
        public long getOrder() {
            return order;
        }

        @Override
        public String getValue() {
            return value;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, order, value, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if(obj == null) {
                return false;
            }
            if (getClass() == obj.getClass()) {
                return false;
            }
            StringValueImpl o = (StringValueImpl) obj;
            return Objects.equals(key, o.key) &&
                    Objects.equals(order, o.order) &&
                    Objects.equals(value, o.value) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("order", order)
                    .add("value", value)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}